/*Robotics 3 - Robot Car
 * Bluetooth control using HC-05 BT module
 * Android app:  Arduino Bluetooth Controlled Joystick, Unica Robotics
 * https://play.google.com/store/apps/details?id=uncia.robotics.joystick&hl=en_US&gl=US
 * 
 * 1.  Parse incoming data string to Angle, Magnitude, and Button state
 * 2.  Convert string data to integer data
 * 3.  Convert polar coordinates to rectangular coordinates
 */

#define AREF 3.33; //connect AREF pin to 3.3V

#define FREQUENCY 3 //blinks per second in units Hz(cycles per second)
#define DUTY_CYCLE 0.25 //ration of ON_TIME to total time per blink or period

#define PERIOD 1000/FREQUENCY //how long the blink takes
#define ON_TIME DUTY_CYCLE * PERIOD //the time the led is on
#define OFF_TIME PERIOD - ON_TIME //the time the led is off

#include "FastLED.h"

#include <SoftwareSerial.h> //You should research what this library does.  You may have to install it if it's not preinstalled

SoftwareSerial MyBluetooth(12, 2); // RX which goes to TX, TX which goes to RX  Replace "pin" with the digital pin you plan to connect to.

char Direction = 'H'; // set default
unsigned long timer;

int analoginput = 5; // our analog pin
int analogamount = 0; // stores incoming value
float percentage = 0; // used to store our percentage value
float voltage =0; // used to store voltage value

// Motor R

int enB = 5;
int in3 = 6;
int in4 = 7;

// Motor L

int enA = 11;
int in1 = 9;
int in2 = 8;

int motorSpeedR = 100;
int motorSpeedL = 100;

CRGB leds[1]; 


// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = 0;        // will store last time LED was updated
boolean On; 
float x_coord, y_coord;
float rad;
int left; // runs left
int right; // runs right
int H = 1; //variable to stop motor

void setup() {
  Serial.begin(115200);
  MyBluetooth.begin(38400);  //Baud Rate for BT control mode (default for HC-05 module)
  //pinMode(enA, OUTPUT);
  //pinMode(enB, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);
  FastLED.addLeds<NEOPIXEL, 13>(leds, 1); //sets pin number, name of the led, and how many 
  FastLED.setBrightness(100);
  leds[0] = CRGB::Black;
  FastLED.show();
  analogReference(EXTERNAL); // use AREF for reference voltage
}

void voltageMonitor(){ //runs voltage monitor
  // check to see if it's time to blink the LED; that is, if the difference
  // between the current time and last time you blinked the LED is bigger than
  // the interval at which you want to blink the LED.
  unsigned long currentMillis = millis();
  analogamount=analogRead(analoginput);
  percentage=(analogamount/1024.00);
  voltage= percentage * (8.4/3.23) * AREF; // in Volts
  
  //Serial.print("A0 (V): ");
  //Serial.println(voltage,2);

  if (voltage >= 7.0) {
      leds[0] = CRGB::Green;
      FastLED.show();
    }

  if (voltage < 7.0 && voltage >= 6.6 ) {
      leds[0] = CRGB::Orange;
      FastLED.show();
    }   

  if (voltage < 6.6){
    if (currentMillis - previousMillis >= OFF_TIME && On == false) {
      previousMillis = currentMillis; //reset clock
 
      leds[0] = CRGB::Red;
      FastLED.setBrightness(30);
      FastLED.show();
      On = true;

    }

    if(currentMillis - previousMillis >= ON_TIME && On == true) {
      previousMillis = currentMillis; //reset clock
      
      leds[0] = CRGB::Black;
      FastLED.setBrightness(30);
      FastLED.show();
      On = false;
    }
       
  }
}



void loop() {

  voltageMonitor();
  
  if (MyBluetooth.available() > 0)
  {
    timer = millis();
    String value = MyBluetooth.readStringUntil('#');
    if (value.length() == 7)
    {
      String angle = value.substring(0, 3);
      String magnitude = value.substring(3, 6);
      String button = value.substring(6, 8);
      /*Serial.print("angle: "); Serial.print(angle); Serial.print('\t');
      Serial.print("magnitude: "); Serial.print(magnitude); Serial.print('\t');
      Serial.print("button: "); Serial.print(button); Serial.print('\t');*/

      int a = angle.toInt();
      int m = magnitude.toInt();

      
   
     
      Serial.flush();
      value = "";
      
      if(a > 0 && a <= 90){ //if the joystick is in the first quadrant, left is determined by magnitude and right is a percentage of left

        left = (m/100.0) * 100.0; 
        right = left - ((90.0 - a) / 90.0) * left;

        digitalWrite(in3, LOW);// forward motor R
        digitalWrite(in4, HIGH);
        digitalWrite(in1, HIGH);// forward motor L
        digitalWrite(in2, LOW);

        left = map(left, 0, 100, 0, 255); //maps 0 to 100 to 0 to 255
        right = map(right, 0, 100, 0, 255);

        analogWrite(enA, right);// Set speed to possible range 0~255
        analogWrite(enB, left);// Set speed to possible range 0~255

        Serial.print("a: "); Serial.print(a); Serial.print('\t');
        Serial.print("m: "); Serial.print(m); Serial.print('\t');

        Serial.print("left: "); Serial.print(left); Serial.print('\t');
        Serial.print("right: "); Serial.print(right); Serial.println();
    
      }

      if(a > 90 && a <= 180){ //if the joystick is in the second quadrant,right is determined by magnitude and left is a percentage of right

        a = a - 90; //make the angles a range between 0 and 90
        
        right = (m/100.0) * 100.0;
        left = right - ((a - 90.0) / 90.0) * right * -1; // angle - 90 so it goes right

        digitalWrite(in3, LOW);// forward motor R
        digitalWrite(in4, HIGH);
        digitalWrite(in1, HIGH);// forward motor L
        digitalWrite(in2, LOW);

        left = map(left, 0, 100, 255, 0); // reverse so the 255 meets at the top of the joystick circle
        right = map(right, 0, 100, 0, 255);

        analogWrite(enA, right);// Set speed to possible range 0~255
        analogWrite(enB, left);// Set speed to possible range 0~255

        Serial.print("a: "); Serial.print(a); Serial.print('\t');
        Serial.print("m: "); Serial.print(m); Serial.print('\t');

        Serial.print("left: "); Serial.print(left); Serial.print('\t');
        Serial.print("right: "); Serial.print(right); Serial.println();
        
    
      }

      if(a > 180 && a <= 270){ //if the joystick is in the third quadrant, right is determined by magnitude and left is a percentage of right

        a = a - 180;
        
        left = (m/100.0) * 100.0;
        right = left - ((a - 90.0) / 90.0) * left * -1;

        digitalWrite(in3, HIGH);// Reverse motor R
        digitalWrite(in4, LOW);
        digitalWrite(in1, LOW);// Reverse motor L
        digitalWrite(in2, HIGH);

        left = map(left, 0, 100, 0, 255);
        right = map(right, 0, 100, 0, 255);

        analogWrite(enA, right);// Set speed to possible range 0~255
        analogWrite(enB, left);// Set speed to possible range 0~255

        Serial.print("a: "); Serial.print(a); Serial.print('\t');
        Serial.print("m: "); Serial.print(m); Serial.print('\t');

        Serial.print("left: "); Serial.print(left); Serial.print('\t');
        Serial.print("right: "); Serial.print(right); Serial.println();
        
      }

      if(a > 270 && a  <= 360){ //if the joystick is in the fourth quadrant, left is determined by magnitude and right is a percentage of left
        
        a = a - 270;
        
        right = (m/100.0) * 100.0;
        left = right - ((90.0 - a) / 90.0) * right;

        digitalWrite(in3, HIGH);// Reverse motor R
        digitalWrite(in4, LOW);
        digitalWrite(in1, LOW);// Reverse motor L
        digitalWrite(in2, HIGH);

        left = map(left, 0, 100, 255, 0);
        right = map(right, 0, 100, 0, 255);

        analogWrite(enA, right);// Set speed to possible range 0~255
        analogWrite(enB, left);// Set speed to possible range 0~255

        Serial.print("a: "); Serial.print(a); Serial.print('\t');
        Serial.print("m: "); Serial.print(m); Serial.print('\t');

        Serial.print("left: "); Serial.print(left); Serial.print('\t');
        Serial.print("right: "); Serial.print(right); Serial.println();

        
        
      }
  
      
    }
     
  }

  else if (millis() - timer > 150) {    // serial read timeout
    H = 0;
  }

  if(H == 0){

        digitalWrite(in3, HIGH);// stop motor R
        digitalWrite(in4, HIGH);
        digitalWrite(in1, HIGH);// stop motor L
        digitalWrite(in2, HIGH);

        left = 0;
        right = 0;

        H = 1;
        
      }
}


  
  
  
